//+------------------------------------------------------------------+
//|                                     StepChartChannel_v2 600+.mq4 |
//|                                Copyright  2014, TrendLaboratory |
//|            http://finance.groups.yahoo.com/group/TrendLaboratory |
//|                                   E-mail: igorad2003@yahoo.co.uk |
//+------------------------------------------------------------------+
#property copyright "Copyright  2014, TrendLaboratory"
#property link      "http://finance.groups.yahoo.com/group/TrendLaboratory"

#property indicator_chart_window
#property indicator_buffers 4
#property indicator_color1 Silver
#property indicator_color2 Silver
#property indicator_color3 DeepSkyBlue
#property indicator_color4 Tomato
#property indicator_width3 2
#property indicator_width4 2

//---- input parameters

extern int     TimeFrame      =   0;   //TimeFrame in min 
extern int     Price          =   0;   //Applied to (0...6)
extern double  StepSize       =  20;   //Size in pips
extern int     HiLoMode       =   0;   //High/Low Mode:0-off,1-on
extern int     VoltyLength    =   0;   //Volty Length(0-will be used StepSize)
extern double  VoltyRatio     =   1;   //Volty Ratio
extern double  Multiplier     =   2;   //Reversal Multiplier 
extern int     PreSmooth      =   1;   //Pre-smoothing Length 
extern int     PreSmoothMode  =   3;   //Pre-smoothing Method  
extern int     StartPriceMode =   0;   //Start Price Mode: 0-as is,1-rounded to StepSize  
extern int     ChannelMode    =   0;   //0-only lines,1-only bricks,2-line & bricks
extern bool   alertsOn         = false;
extern bool   alertsOnCurrent  = false;
extern bool   alertsMessage    = true;
extern bool   alertsSound      = false;
extern bool   alertsEmail      = false;

//---- indicator buffers

double upband[];
double loband[];
double uptrend[];
double dntrend[];
double trend[];



double   line[2], _point;
datetime lineprevtime;
string   TF, IndicatorName;
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
int init()
{
   if(TimeFrame <= Period()) TimeFrame = Period();
   IndicatorDigits(Digits);

//---- indicator line
   IndicatorBuffers(5);
        
   SetIndexBuffer(0, upband); SetIndexStyle(0,     DRAW_LINE);
   SetIndexBuffer(1, loband); SetIndexStyle(1,     DRAW_LINE);
   SetIndexBuffer(2,uptrend); SetIndexStyle(2,DRAW_HISTOGRAM);
   SetIndexBuffer(3,dntrend); SetIndexStyle(3,DRAW_HISTOGRAM);
   SetIndexBuffer(4,  trend);

//---- name for DataWindow and indicator subwindow label
   switch(TimeFrame)
   {
   case 1     : TF = "M1" ; break;
   case 5     : TF = "M5" ; break;
   case 15    : TF = "M15"; break;
   case 30    : TF = "M30"; break;
   case 60    : TF = "H1" ; break;
   case 240   : TF = "H4" ; break;
   case 1440  : TF = "D1" ; break;
   case 10080 : TF = "W1" ; break;
   case 43200 : TF = "MN1"; break;
   } 
   
   IndicatorName = WindowExpertName();
   string short_name = IndicatorName+"["+TF+"]("+Price+","+DoubleToStr(StepSize,2)+","+HiLoMode+","+VoltyLength+","+DoubleToStr(VoltyRatio,1)+","+DoubleToStr(Multiplier,1)+")";
   IndicatorShortName(short_name);
   SetIndexLabel(0,"UpBand"+"["+TF+"]");
   SetIndexLabel(1,"DnBand"+"["+TF+"]");
   SetIndexLabel(2,"UpTrend"+"["+TF+"]");
   SetIndexLabel(3,"DnTrend"+"["+TF+"]");
//----
   int begin = MathMax(2,Bars - iBars(NULL,TimeFrame)*TimeFrame/Period() + VoltyLength + PreSmooth); 
   SetIndexDrawBegin(0,begin);
   SetIndexDrawBegin(1,begin);
   SetIndexDrawBegin(2,begin);
   SetIndexDrawBegin(3,begin);
//----
   
   _point = MarketInfo(Symbol(),MODE_POINT) * MathPow(10,Digits%2);
   
//----
   return(0);
}

//+------------------------------------------------------------------+
//| StepChartChannel_v2 600+                                         |
//+------------------------------------------------------------------+
int start()
{
   int shift,limit, counted_bars = IndicatorCounted();
    
   if(counted_bars > 0) limit = Bars - counted_bars - 1;
   if(counted_bars < 0) return(0);
   if(counted_bars < 1)
	{
   limit = Bars - 1;   
      for(int i=limit;i>=0;i--) 
      {
      upband[i]  = EMPTY_VALUE;
      loband[i]  = EMPTY_VALUE;
      uptrend[i] = EMPTY_VALUE;
      dntrend[i] = EMPTY_VALUE;
      }   
   }
	
   if(TimeFrame != Period())
	{
   limit = MathMax(limit,TimeFrame/Period());   
      
      for(shift = 0;shift < limit;shift++) 
      {	
      int y = iBarShift(NULL,TimeFrame,Time[shift]);
      upband[shift]  = iCustom(NULL,TimeFrame,IndicatorName,0,Price,StepSize,HiLoMode,VoltyLength,VoltyRatio,Multiplier,PreSmooth,PreSmoothMode,StartPriceMode,ChannelMode,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,0,y);   
      loband[shift]  = iCustom(NULL,TimeFrame,IndicatorName,0,Price,StepSize,HiLoMode,VoltyLength,VoltyRatio,Multiplier,PreSmooth,PreSmoothMode,StartPriceMode,ChannelMode,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,1,y);   
      uptrend[shift] = iCustom(NULL,TimeFrame,IndicatorName,0,Price,StepSize,HiLoMode,VoltyLength,VoltyRatio,Multiplier,PreSmooth,PreSmoothMode,StartPriceMode,ChannelMode,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,2,y);   
      dntrend[shift] = iCustom(NULL,TimeFrame,IndicatorName,0,Price,StepSize,HiLoMode,VoltyLength,VoltyRatio,Multiplier,PreSmooth,PreSmoothMode,StartPriceMode,ChannelMode,alertsOn,alertsOnCurrent,alertsMessage,alertsSound,alertsEmail,3,y);   
      }  
	return(0);
	}   
   else _StepChartChannel(limit,counted_bars);
   
   return(0);	
}

void _StepChartChannel(int limit,int counted_bars)
{
   double volty;
     
   for(int shift = limit;shift >= 0;shift--) 
   {	
      if(lineprevtime != Time[shift])
      {
      line[1]      = line[0];   
      lineprevtime = Time[shift]; 
      }   
   
   
   line[0] = StepPrice(Price,StepSize,HiLoMode,VoltyLength,VoltyRatio,Multiplier,PreSmooth,PreSmoothMode,StartPriceMode,volty,shift);    
     
   trend[shift] = trend[shift+1];               
      
      if(trend[shift] != 0)
      {
      if(trend[shift] > 0) trend[shift] += 1; 
      if(trend[shift] < 0) trend[shift] -= 1; 
      upband[shift]  = upband[shift+1];
      loband[shift]  = loband[shift+1];   
      uptrend[shift] = uptrend[shift+1];
      dntrend[shift] = dntrend[shift+1];
      }   
      
      if(line[0] > line[1]) 
      {
      if(trend[shift] <= 0) trend[shift] = 1;
      
         if(ChannelMode == 0)
         {
         upband[shift]  = line[0];
         loband[shift]  = line[0] - volty;
         uptrend[shift] = EMPTY_VALUE;
         dntrend[shift] = EMPTY_VALUE;
         }
         else
         if(ChannelMode == 1)
         {
         uptrend[shift] = line[0];
         dntrend[shift] = line[0] - volty;
         }
         else
         {
         upband[shift]  = line[0];
         loband[shift]  = line[0] - volty;
         uptrend[shift] = line[0];
         dntrend[shift] = line[0] - volty;
         }
      }    
   
      if(line[0] < line[1]) 
      {
      if(trend[shift] >= 0) trend[shift] =-1;    
         
         if(ChannelMode == 0)
         {
         upband[shift]  = line[0] + volty;
         loband[shift]  = line[0];
         }
         else
         if(ChannelMode == 1)
         {
         dntrend[shift] = line[0] + volty;
         uptrend[shift] = line[0];
         }
         else
         {
         upband[shift]  = line[0] + volty;
         loband[shift]  = line[0];
         dntrend[shift] = line[0] + volty;
         uptrend[shift] = line[0];
         }
      }
   }      
   manageAlerts();
}   


//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

void manageAlerts()
{
   if (alertsOn)
   {
      if (alertsOnCurrent)
           int whichBar = 0;
      else     whichBar = 1;
      if (trend[whichBar] != trend[whichBar+1])
      {
         if (trend[whichBar] ==  1) doAlert(whichBar,"up");
         if (trend[whichBar] == -1) doAlert(whichBar,"down");
      }
   }
}

//
//
//
//
//

void doAlert(int forBar, string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
   if (previousAlert != doWhat || previousTime != Time[forBar]) {
       previousAlert  = doWhat;
       previousTime   = Time[forBar];

       //
       //
       //
       //
       //

       message =  StringConcatenate(Symbol()," at ",TimeToStr(TimeLocal(),TIME_SECONDS)," ",timeFrameToString(Period())+" StepChart trend changed to ",doWhat);
          if (alertsMessage) Alert(message);
          if (alertsEmail)   SendMail(StringConcatenate(Symbol(),"StepChart trend "),message);
          if (alertsSound)   PlaySound("alert2.wav");
   }
}

//+-------------------------------------------------------------------
//|                                                                  
//+-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//----

int    stepprevtime, dir[2];
double sprice[2];

double StepPrice(int pricemode,double size,int hilo,int voltylen,double voltyr,double mult,int presm,int premode,int start,double& step,int bar)
{
   if(stepprevtime != Time[bar])
   {
   sprice[1]    = sprice[0];   
   dir[1]       = dir[0]; 
   stepprevtime = Time[bar]; 
   }
   
   if(voltylen > 0 && bar < Bars - MathMax(1,presm + voltylen))
   {
   double sum  = 0;
      
      if(hilo == 0)
      {
      for(int i=0;i<voltylen;i++) 
         sum += MathAbs(iMA(NULL,0,presm,0,premode,pricemode,bar+i) - iMA(NULL,0,presm,0,premode,pricemode,bar+1+i));
      }
      else
      {
      for(i=0;i<voltylen;i++) 
         sum += iMA(NULL,0,presm,0,premode,2,bar+i) - iMA(NULL,0,presm,0,premode,3,bar+i);
      }        
      
   if(size > 0) step = MathMax(size,MathRound(voltyr*sum/voltylen/_point))*_point;
   else step = MathMax(0.1,MathRound(voltyr*sum/voltylen/_point))*_point;
   }
   else 
   if(voltylen == 0) step = size*_point;
   
   
   step = NormalizeDouble(step,Digits);
    
      if(hilo == 0)
      {
      double hiprice = NormalizeDouble(iMA(NULL,0,presm,0,premode,pricemode,bar),Digits);
      double loprice = hiprice;
      }
      else
      {
      hiprice = NormalizeDouble(iMA(NULL,0,presm,0,premode,2,bar),Digits);
      loprice = NormalizeDouble(iMA(NULL,0,presm,0,premode,3,bar),Digits);
      }
  
   if(bar < Bars - MathMax(1,presm+voltylen))
   {
   sprice[0] = sprice[1];   
   dir[0]    = dir[1]; 
   
   if(dir[0] != 0) double value = NormalizeDouble(mult*step,Digits);
   else value  = NormalizeDouble(step,Digits);
   
   double price = sprice[0];
   if(hiprice > sprice[0]) price = hiprice;
   else
   if(loprice < sprice[0]) price = loprice;
      
   double hidel  = NormalizeDouble(price - sprice[0],Digits);
   double lodel  = NormalizeDouble(sprice[0] - price,Digits);
         
      if(dir[0] <= 0 && hidel < value && hidel >= 0) sprice[0] = sprice[1];
      else
      if(dir[0] >= 0 && lodel < value && lodel >= 0) sprice[0] = sprice[1];          
      else
      { 
         while(NormalizeDouble(MathAbs(price - sprice[0]),Digits) >= step) 
         {
    	      if(price > sprice[0]) sprice[0] = NormalizeDouble(sprice[0] + step,Digits);
		      if(price < sprice[0]) sprice[0] = NormalizeDouble(sprice[0] - step,Digits);
		   }
		}    
    
   double del = NormalizeDouble(sprice[0] - sprice[1],Digits);
                 
   if(del  >= value && dir[0] <= 0) dir[0] = 1;
   if(-del >= value && dir[0] >= 0) dir[0] =-1;    
         
   
   }
   else
   if(bar == Bars - MathMax(1,presm+voltylen))
   { 
      if(start == 0) sprice[0] = NormalizeDouble(iMA(NULL,0,presm,0,premode,pricemode,bar),Digits);
      else
      {
      if(step != 0.0) sprice[0] = MathRound(price/step) * step; else sprice[0] = price;         
      }
   
   dir[0] = 0;
   }
   return(sprice[0]);
}  
     